/* Gutter Icons */

// valid gutter icon identifiers
// the values = title = tooltip for the icon
var gutter_icons = {
	'gear' : 'Settings',
	'timer' : 'Add Timer',
	'timestamp' : 'Add Timestamp',
	'checkmark' : 'Add Checkmark',
	'timestamp_end' : 'Stop and add final timestamp',
	'timer_start' : 'Start timer',
	'timer_pause' : 'Pause timer',
	'timer_resume' : 'Resume timer'
}

function add_icon_to_gutter(icon_identifier, gutter_div)
{
	if ( ! gutter_icons[icon_identifier] )
		die_because_icon_identifier_is_not_valid();
	
	// make sure it does not already exist
	var children = gutter_div.childNodes
	for ( index in children ) {
		child = children[index]
		if ( child.className && child.className.match(/gutter_icon/) && child.className.match(icon_identifier) )
			return
	}
	
	// create new div child with the right settings
	var gutter_icon_div = document.createElement('div')
	gutter_icon_div.className = 'gutter_icon ' + icon_identifier
	gutter_icon_div.setAttribute('onclick', 'did_click_gutter_' + icon_identifier + '(event);')
	gutter_icon_div.title = gutter_icons[icon_identifier]
	
	// add to gutter div
	gutter_div.appendChild(gutter_icon_div)
}

function remove_icon_from_gutter(icon_identifier, gutter_div)
{
	if ( ! gutter_icons[icon_identifier] )
		die_because_icon_identifier_is_not_valid();
	
	// find the gutter_icon div in the children
	var children = gutter_div.childNodes
	for ( index in children ) {
		child = children[index]
		if ( child.className && child.className.match(/gutter_icon/) && child.className.match(icon_identifier) ) {
			// remove from parent div
			gutter_div.removeChild(child)
			return
		}
	}
}

function remove_all_icons_from_gutter(gutter_div)
{
	// iterate over the gutter_icon divs in the children
	var children = gutter_div.childNodes
	for ( index in children ) {
		child = children[index]
		if ( child.className && child.className.match(/gutter_icon/) )
			gutter_div.removeChild(child)
	}
}


function gutter_child_div(gutter_div, subdiv_class_name)
{
	// make sure it does not already exist
	var children = gutter_div.childNodes
	for ( index in children ) {
		child = children[index]
		if ( child.className && child.className.match(subdiv_class_name) )
			return child
	}
	return null
}

/* Gutter Text */

function set_gutter_text_to_string(gutter_div, gutter_string, tooltip, text_state)
{
	if ( ! gutter_div )
		return;
	var gutter_text_div = gutter_child_div(gutter_div,'gutter_text')
	
	// remove gutter_text div if no string
	if ( ! gutter_string ) {
		if ( gutter_text_div )
			gutter_div.removeChild(gutter_child_div(gutter_div,'gutter_text'))
		return
	}
	
	// create gutter_text div if necessary
	if ( ! gutter_text_div ) {
		gutter_text_div = document.createElement('div')
		gutter_text_div.className = 'gutter_text'
		gutter_text_div.setAttribute('onclick', 'did_click_gutter_text(event);')
		gutter_div.appendChild(gutter_text_div)
	}
	
	// set the string using the innerHTML property
	gutter_text_div.innerHTML = gutter_string
	
	if ( tooltip )
		gutter_text_div.title = tooltip
	else
		gutter_text_div.title = ''
	
	// special glowing effect
	var alternate_tag = ''
	if ( text_state == 'finished' && ( ! gutter_text_div.className.match(/alternate/) ) )
			alternate_tag = ' alternate'
	
	// text_state is set using the class attribute
	gutter_text_div.className = 'gutter_text ' + text_state + alternate_tag
}

function update_gutter_contents(gutter_div)
{
	// current status/info
	var start = new TimeStamp(gutter_div.getAttribute('timestamp_start'))
	var end = new TimeStamp(gutter_div.getAttribute('timestamp_end'))
	var timer_status = gutter_div.getAttribute('timer_status')
	var timer_duration = new TimeStamp(gutter_div.getAttribute('timer_duration'))
	var timer_time_left = new TimeStamp(gutter_div.getAttribute('timer_time_left'))
	var display_attribute = gutter_div.getAttribute('display')

	// values we need to determine by the end of the function
	var gutter_string = ''
	var tooltip = ''
	var text_state = 'inactive'

	// determine mode based on the different fields
	var gutter_mode = 'none'
	if ( timer_duration.isDefined() )
		gutter_mode = 'timer'
	else if ( start.isDefined() )
		gutter_mode = 'timestamp'
	else
		gutter_mode = 'none'
	//gutter_div.setAttribute('mode', gutter_mode)
	remove_all_icons_from_gutter(gutter_div)
	add_icon_to_gutter('gear', gutter_div)

	// mode: none
	if ( gutter_mode == 'none' ) {
		remove_all_icons_from_gutter(gutter_div)
		add_icon_to_gutter('timer', gutter_div)
		add_icon_to_gutter('timestamp', gutter_div)
		var paragraph = gutter_div.parentNode
		var container_body = paragraph.parentNode
		var container = container_body.parentNode
		var p = paragraph.getElementsByTagName('p')[0]
		if ( p.className.match(/bullet/) )
			add_icon_to_gutter('checkmark', gutter_div)
	}

	// mode : timestamp 
	else if ( gutter_mode == 'timestamp' ) {
		
		// display : duration
		if ( display_attribute == 'duration' ) {
			tooltip = 'Click to display start time'
			var duration = null
			// only start is defined --> time elapsed between start and now
			if ( start.isDefined()  &&  ( ! end.isDefined() ) ) {
				var now = new TimeStamp()
				gutter_string = now.minusTimeStamp(start).timerString()
				register_gutter_for_updates(gutter_div)
				add_icon_to_gutter('timestamp_end', gutter_div)
				text_state = 'running'
			}
			// both start and end are defined --> time elapsed between start and end
			else if ( start.isDefined() &&  end.isDefined() ) {
				gutter_string = end.minusTimeStamp(start).durationString()
				unregister_gutter_for_updates(gutter_div)
				remove_icon_from_gutter('timestamp_end', gutter_div)
			}
		}

		// display : start
		else if ( display_attribute == 'start' ) {
			gutter_string = start.shortTimeString()
			if ( end.isDefined() ) {
				var duration = end.minusTimeStamp(start)
				if ( duration.minutes() < 5 && duration.hours() < 1)
					tooltip = 'Click to display start time and time elapsed'
				else
					tooltip = 'Click to display start and end time'
			}
			else
				tooltip = 'Click to display time elapsed'
			unregister_gutter_for_updates(gutter_div)
			remove_icon_from_gutter('timestamp_end', gutter_div)
		}

		// display : span
		else if ( display_attribute == 'span' ) {
			tooltip = 'Click to display time elapsed'

			// only start is defined --> only display start (should not happen)
			if ( start.isDefined() == true &&  end.isDefined() == false ) {
				gutter_string = start.shortTimeString()
				register_gutter_for_updates(gutter_div)
				add_icon_to_gutter('timestamp_end', gutter_div)
			}

			// both start and end are defined --> time elapsed between start and end
			else if ( start.isDefined() == true &&  end.isDefined() == true ) {
				var duration = end.minusTimeStamp(start)
				if ( duration.minutes() < 5 && duration.hours() < 1)
					// show duration for small intervals
					gutter_string = start.shortTimeString() + '-' + duration.durationString()
				else
					// show start and end for long intervals
					gutter_string = start.shortTimeString() + '-' + end.shortTimeString()
				unregister_gutter_for_updates(gutter_div)
				remove_icon_from_gutter('timestamp_end', gutter_div)
			}
		}
	}

	// mode : timer 
	else if ( gutter_mode == 'timer' ) {

/* 

This is an overview of the different states a timer might be in, and the actions we need to take to adjust the different parameters.

	'X' --> the value is set, we don't change it
	'–' --> the value is not set, we don't change it
	'+' --> we change the value based on the other parameters
	'o' --> we change the value to undef or zero

                     time    time    time    status=   status=  status=      
                     start    end    left    paused   finished  stopped      
                                                                             
pending non-sched.     –       o       +        o        o         o         
                                                                             
pending scheduled      X       +       +        o        o         o         
                                                                             
running                X       X       +        –        –         –         
                                                                             
paused                 X       +       X        X        –         –         
                                                                             
finished               X       X       o        –        +         –         
                                                                             
stopped                X       X       o        –        –         X         

*/

		// there should always be a timer_duration property
		if ( timer_duration.isDefined() == false )
			timer_duration = new TimeStamp('00:05:00')
		
		// derived values
		var now = new TimeStamp()
		var computed_time_elapsed = now.minusTimeStamp(start)
		var computed_time_left = null;
		if ( timer_time_left.isDefined() )
			computed_time_left = timer_time_left
		else if ( end.isDefined() )
			computed_time_left = end.minusTimeStamp(now)
		else if ( start.isDefined() )
			computed_time_left = start.plusTimeStamp(timer_duration).minusTimeStamp(now)
		else
			computed_time_left = timer_duration

		// start == undef
		// timer is pending and is not scheduled
		if ( start.isDefined() == false ) {
			gutter_div.removeAttribute('timestamp_end')
			gutter_div.setAttribute('timer_time_left', timer_duration.timeString())
			gutter_div.setAttribute('timer_status', 'pending')
		}

		// now < start
		// timer is pending and is scheduled
		else if ( computed_time_elapsed.isDefined() == false ) {
			end = start.plusTimeStamp(timer_duration)
			gutter_div.setAttribute('timestamp_end', end.timeString())
			gutter_div.setAttribute('timer_time_left', timer_duration.timeString())
			gutter_div.setAttribute('timer_status', 'pending')
			var how_soon = start.minusTimeStamp(now)
			// when the time comes, we should start the timer
			var seconds_before_starting = how_soon.seconds() + 60 * how_soon.minutes() + 3600 * how_soon.hours()
			var milliseconds = 1000.0 * seconds_before_starting
			if ( seconds_before_starting > 10 )
				milliseconds /= 2.0
			else
				milliseconds += 1000.0
			setTimeout( function(){register_gutter_for_updates(gutter_div);}, milliseconds )
		}

		// start  <  now  <  end
		// timer has started and is running
		else if ( computed_time_left.isDefined() == true && timer_status != 'paused' ) {
			// end should be defined at this point, but let's handle the case anyway
			if ( end.isDefined() == false ) {
				end = start.plusTimeStamp(timer_duration)
				gutter_div.setAttribute('timestamp_end', end.timeString())
			}
			timer_time_left = end.minusTimeStamp(now)
			gutter_div.setAttribute('timer_time_left', timer_time_left.timeString())
			gutter_div.setAttribute('timer_status', 'running')
		}

		// start  <  now  <  end
		// timer has started and is paused
		else if ( computed_time_left.isDefined() == true && timer_status == 'paused' ) {
			// timer_time_left should be defined at this point, but let's handle the case anyway
			if ( timer_time_left.isDefined() == false ) {
				timer_time_left = start.plusTimeStamp(timer_duration).minusTimeStamp(now)
				gutter_div.setAttribute('timer_time_left', timer_time_left.timeString())
			}
			end = now.plusTimeStamp(timer_time_left)
			gutter_div.setAttribute('timestamp_end', end.timeString())
		}

		// end  <  now
		// timer is finished / stopped 
		else {
			// end should be defined at this point, but let's handle the case anyway
			if ( end.isDefined() == false ) {
				end = start.plusTimeStamp(timer_duration)
				gutter_div.setAttribute('timestamp_end', end.timeString())
			}
			gutter_div.removeAttribute('timer_time_left')
			// status == stopped --> means the user has not stopped it yet
			if ( timer_status != 'stopped' )
				gutter_div.setAttribute('timer_status', 'finished')
		}
		

		// now ready for changing the gutter contents
		
		// reload the status/info for the timer, as some may have changed
		end = new TimeStamp(gutter_div.getAttribute('timestamp_end'))
		timer_time_left = new TimeStamp(gutter_div.getAttribute('timer_time_left'))
		timer_status = gutter_div.getAttribute('timer_status')

/*
These are the possible 'display_attribute' values for the different 'timer_status' values:

                     status:   status:   status:   status:   status:
                     pending   running   paused    finished  stopped
display:duration       –         X         X         –         X    
display:countdown      X         X         X         X         –    
display:start          –         –         –         –         X    
display:span           –         –         –         –         X    
*/

		// adjust display attribute if necessary
		if ( timer_status == 'pending' || timer_status == 'finished' )
			display_attribute = 'countdown'
		else if ( timer_status != 'stopped' ) {
			if ( display_attribute != 'duration' && display_attribute != 'countdown')
				display_attribute = 'countdown'
		} else if ( display_attribute == 'countdown' )
			display_attribute = 'duration'
		gutter_div.setAttribute('display', display_attribute)

		// now we can go through the different cases to adjust the gutter contents
		
		// automatic timer updates only necessary when running
		if ( timer_status == 'running' || timer_status == 'paused' || timer_status == 'finished' )
			register_gutter_for_updates(gutter_div)
		else
			unregister_gutter_for_updates(gutter_div)
		
		// tooltip --> shows the next display option
		tooltip = 'Click to display time left'
		if ( timer_status == 'pending' )
			tooltip = ''
		else if ( timer_status == 'finished' )
			tooltip = 'Click to stop and display time elapsed'
		else if ( display_attribute == 'countdown' )
			tooltip = 'Click to display time elapsed'
		else if ( display_attribute == 'duration' && timer_status == 'stopped' )
			tooltip = 'Click to display start time'
		else if ( display_attribute == 'start' )
			tooltip = 'Click to display start time and duration'
		else if ( display_attribute == 'span' )
			tooltip = 'Click to display duration'
		
		// display : duration  -->  = time elapsed
		if ( display_attribute  == 'duration' ) {
			if ( timer_status == 'stopped' )
				gutter_string = end.minusTimeStamp(start).durationString()
			else
				gutter_string = computed_time_elapsed.timerString()
		}

		// display : countdown
		else if (  gutter_div.getAttribute('display') == 'countdown' ) {
			if ( timer_status == 'finished' )
				gutter_string = '-' + now.minusTimeStamp(end).timerString()
			else
				gutter_string = timer_time_left.timerString()
		}
		
		// display : start
		else if (  gutter_div.getAttribute('display') == 'start' )
			gutter_string = start.shortTimeString()

		// display : span
		else if (  gutter_div.getAttribute('display') == 'span' ) {
			var duration = end.minusTimeStamp(start)
			if ( duration.minutes() < 5 && duration.hours() < 1)
				// show duration for small intervals
				gutter_string = start.shortTimeString() + '-' + duration.durationString()
			else
				// show start and end for long intervals
				gutter_string = start.shortTimeString() + '-' + end.shortTimeString()
		}
		
		// 'start' icon
		if ( timer_status == 'pending' )
			add_icon_to_gutter('timer_start', gutter_div)
		else
			remove_icon_from_gutter('timer_start', gutter_div)

		// 'pause' icon
		if ( timer_status == 'running' )
			add_icon_to_gutter('timer_pause', gutter_div)
		else
			remove_icon_from_gutter('timer_pause', gutter_div)

		// 'resume' icon
		if (timer_status == 'paused' )
			add_icon_to_gutter('timer_resume', gutter_div)
		else
			remove_icon_from_gutter('timer_resume', gutter_div)

		// text state --> changes background color
		text_state = timer_status
	}
	
	set_gutter_text_to_string(gutter_div, gutter_string, tooltip, text_state)
}

function did_click_gutter_text()
{
	var gutter_icon_div = event.target
	var gutter_div = gutter_icon_div.parentNode
	var paragraph = gutter_div.parentNode
	var container_body = paragraph.parentNode
	var container = container_body.parentNode

	// determine mode based on the different fields
	var start = new TimeStamp(gutter_div.getAttribute('timestamp_start'))
	var end = new TimeStamp(gutter_div.getAttribute('timestamp_end'))
	var timer_duration = new TimeStamp(gutter_div.getAttribute('timer_duration'))
	var gutter_mode = gutter_div.getAttribute('mode')
	if ( timer_duration.isDefined() )
		gutter_mode = 'timer'
	else if ( start.isDefined() )
		gutter_mode = 'timestamp'
	else
		gutter_mode = 'none'

	if ( gutter_mode == 'timestamp' ) {
		
		var current_display = gutter_div.getAttribute('display')
		
		// display:duration --> toggle to display:start
		if ( current_display == 'duration' )
			gutter_div.setAttribute('display', 'start')
		
		// display:start --> toggle to display:span or display:duration
		else if ( current_display == 'start' ) {

			// if there is also an 'end' timestamp, we allow display:span
			if ( gutter_div.getAttribute('timestamp_end'))
				gutter_div.setAttribute('display', 'span')

			// otherwise back the duration display with updates
			else
				gutter_div.setAttribute('display', 'duration')
		}
		
		// display:span --> toggle to display:duration
		else if ( current_display == 'span' )
			gutter_div.setAttribute('display', 'duration')
		
	}

	else if ( gutter_mode == 'timer' ) {

/*
These are the possible 'display_attribute' values for the different 'timer_status' values:

                     status:   status:   status:   status:   status:
                     pending   running   paused    finished  stopped
display:duration       –         X         X         X         X    
display:countdown      X         X         X         X         –    
display:start          –         –         –         –         X    
display:span           –         –         –         –         X    
*/
		
		var current_display = gutter_div.getAttribute('display')
		var timer_status = gutter_div.getAttribute('timer_status')
		var new_display = 'countdown'
		
		if (  timer_status == 'finished') {
			new_display = 'duration'
			gutter_div.setAttribute('timer_status', 'stopped')
		}
		else if ( current_display == 'countdown' && timer_status != 'pending' )
			new_display = 'duration'

		else if ( current_display == 'duration' && timer_status == 'stopped' )
				new_display = 'start'
		
		else if ( current_display == 'start' )
			new_display = 'span'
		
		else if ( current_display == 'span' )
			new_display = 'duration'
		
		gutter_div.setAttribute('display', new_display)
		update_gutter_contents(gutter_div)
	}
	
	update_gutter_contents(gutter_div)

}

// keep track of gutter divs that need to be updated for timer or elapsed time

var active_gutter_divs = new Array()
var gutter_divs_to_add = new Array()
var gutter_divs_to_remove = new Array()
var is_running_timers = false
var is_updating_timers = false

function register_gutter_for_updates(gutter_div)
{
	gutter_divs_to_add.push(gutter_div)
	run_timers()
}

function unregister_gutter_for_updates(gutter_div)
{
	gutter_divs_to_remove.push(gutter_div)
	run_timers()
}

function run_timers()
{
	if ( is_running_timers == true )
		return
	is_running_timers = true
	update_timers()
}

function stop_timers()
{
	is_running_timers = false
}

// update the array of active divs
// - start: active_gutter_divs
// - minus: gutter_divs_to_remove
// - plus:  gutter_divs_to_add
// - final: --> active_gutter_divs updated
function update_array_of_active_gutter_divs()
{
	var final_array = new Array()
	for ( var i in active_gutter_divs ) {
		var div1 = active_gutter_divs[i]
		var should_keep = true
		for ( var j in gutter_divs_to_remove ) {
			var div2 = gutter_divs_to_remove[j]
			if ( div1 == div2 )
				should_keep = false
		}
		if ( should_keep )
			final_array.push(div1)
	}
	for ( var i in gutter_divs_to_add ) {
		var div1 = gutter_divs_to_add[i]
		var should_add = true
		for ( var j in final_array ) {
			var div2 = final_array[j]
			if ( div1 == div2 )
				should_add = false
		}
		if ( should_add )
			final_array.push(div1)
	}
	gutter_divs_to_add = new Array()
	gutter_divs_to_remove = new Array()
	active_gutter_divs = final_array
}


function update_timers()
{
	if ( is_running_timers == false )
		return
	
	// avoid reentrant calls
	if ( is_updating_timers )
		return
	is_updating_timers = true
	
	// update gutters
	update_array_of_active_gutter_divs()
	for ( index in active_gutter_divs ) {
		var gutter_div = active_gutter_divs[index]
		update_gutter_contents(gutter_div)
	}

	// for debugging: print out the number of timers running
	var timer_info = document.getElementById('timer_info')
	if ( timer_info ) {
		timer_info.innerHTML = "" + active_gutter_divs.length
		// timer_info.className = timer_info.className + ' temporary'
		// timer_info.className = timer_info.className.replace(' temporary temporary','')
	}

	// schedule next update in 1 second
	if ( active_gutter_divs.length < 1 )
		is_running_timers = false;
	else {
		is_running_timers = true
		setTimeout(update_timers, 1000);
	}

	// end of reentrance barrier
	is_updating_timers = false

}


/* Gutter Settings */

function open_settings_for_gutter(gutter_div)
{
	if ( ! gutter_div )
		return;

	// exit if already open at the same gutter_div
	var settings_div = document.getElementById('gutter_settings')
	if ( settings_div != null && settings_div.parentNode == gutter_div )
		return;
	
	// close gutter settings if already open (and that would be on another gutter div!)
	close_gutter_settings()
	
	// create the settings div from scratch
	// the settings are implemented as a subdiv of gutter_div
	// the created div will be unique in the entire page, with id 'gutter_settings'

	// the settings icon is the gear icon and is used to close the settings panel
	var settings_icon_div = document.createElement('div')
	settings_icon_div.id = 'gutter_settings_icon'
	settings_icon_div.onclick = function() { close_gutter_settings(); }
	
	// text fields = values that can be set for each of the radio button options
	var timefields_description = [ 
		{ 'identifier' : 'timer_duration'    , 'label' : 'Duration:'   },
		{ 'identifier' : 'timestamp_start'   , 'label' : 'Start Time:' },
		{ 'identifier' : 'timestamp_end'     , 'label' : 'End Time:'   }
	]
	var timefields_div = document.createElement('div')
	timefields_div.className = 'settings_timefields'
	for ( field_index in [0,1,2] ) {

		// description from the hash
		var identifier = timefields_description[field_index]['identifier']
		var label_string = timefields_description[field_index]['label']

		// label for the time field
		var label_element = document.createElement('label')
		label_element.className = 'settings_time ' + identifier
		label_element.id = 'settings_time_label_' + identifier
		label_element.setAttribute('for', 'settings_time_' + identifier)
		label_element.appendChild(document.createTextNode(label_string))
		timefields_div.appendChild(label_element)
		
		// each input = 3 text field for the hh, mm, ss
		var time_input = document.createElement('div')
		time_input.className = 'time_input ' + identifier
		time_input.id = 'time_input_' + identifier
		var all_fields = new Array('hours','mins','secs')
		while ( field_identifier = all_fields.shift() ) {
			var textfield = document.createElement('input')
			textfield.type = 'text'
			textfield.className = 'settings_time ' + identifier + ' ' + field_identifier
			textfield.id = identifier  + '_' + field_identifier
			textfield.value = '--'
			textfield.onclick = function() {
				// forces selection of the whole field before editing
				this.focus(); this.select();
			}
			textfield.onblur = function() {
				// formats entry to '00', '01', ..., '99' or '--'
				var final_value = this.value
				if ( final_value == '' )
					final_value = '--'
				var final_value = Math.floor(0+final_value)
				if ( final_value > 99 )
					final_value = 0
				if ( isNaN(final_value) )
					final_value = '--'
				else
					final_value = (final_value<10?"0":"") + final_value
				this.value = final_value
			}
			time_input.appendChild(textfield)
			// the ":" string as a separator = 'hh:mm:ss'
			if ( all_fields.length > 0 )
				time_input.appendChild(document.createTextNode(':'))
		}
		timefields_div.appendChild(time_input)

	}

	// 'now' button for the 'Start' timestamp
	var now1_button = document.createElement('div')
	now1_button.className = 'now_timestamp_start_button'
	now1_button.id = 'now_timestamp_start_button'
	now1_button.title = 'Set timestamp to current time'
	now1_button.onclick = function() {
		var now = new TimeStamp()
		document.getElementById('timestamp_start_hours').value = now.hoursString()
		document.getElementById('timestamp_start_mins').value = now.minutesString()
		document.getElementById('timestamp_start_secs').value = now.secondsString()
	}
	timefields_div.appendChild(now1_button)

	// 'reset' button for the 'Start' timestamp
	var reset1_button = document.createElement('div')
	reset1_button.className = 'reset_timestamp_start_button'
	reset1_button.id = 'reset_timestamp_start_button'
	reset1_button.title = 'Remove timestamp (double click to clear all fields)'
	reset1_button.onclick = function() {
		document.getElementById('timestamp_start_hours').value = '--'
		document.getElementById('timestamp_start_mins').value = '--'
		document.getElementById('timestamp_start_secs').value = '--'
	}
	reset1_button.ondblclick = function() {
		// reset all fields
		document.getElementById('timestamp_start_hours').value = '--'
		document.getElementById('timestamp_start_mins').value = '--'
		document.getElementById('timestamp_start_secs').value = '--'
		document.getElementById('timestamp_end_hours').value = '--'
		document.getElementById('timestamp_end_mins').value = '--'
		document.getElementById('timestamp_end_secs').value = '--'
		document.getElementById('timer_duration_hours').value = '--'
		document.getElementById('timer_duration_mins').value = '--'
		document.getElementById('timer_duration_secs').value = '--'
	}
	timefields_div.appendChild(reset1_button)

	// 'now' button for the 'End' timestamp
	var now2_button = document.createElement('div')
	now2_button.className = 'now_timestamp_end_button'
	now2_button.id = 'now_timestamp_end_button'
	now2_button.title = 'Set timestamp to current time'
	now2_button.onclick = function() {
		var now = new TimeStamp()
		document.getElementById('timestamp_end_hours').value = now.hoursString()
		document.getElementById('timestamp_end_mins').value = now.minutesString()
		document.getElementById('timestamp_end_secs').value = now.secondsString()
	}
	timefields_div.appendChild(now2_button)

	// 'reset' button for the 'End' timestamp
	var reset2_button = document.createElement('div')
	reset2_button.className = 'reset_timestamp_end_button'
	reset2_button.id = 'reset_timestamp_end_button'
	reset2_button.title = 'Remove timestamp (double click to clear all fields)'
	reset2_button.onclick = function() {
		document.getElementById('timestamp_end_hours').value = '--'
		document.getElementById('timestamp_end_mins').value = '--'
		document.getElementById('timestamp_end_secs').value = '--'
	}
	reset2_button.ondblclick = reset1_button.ondblclick
	timefields_div.appendChild(reset2_button)
	
	// 'reset' button for the 'Duration' field
	var reset3_button = document.createElement('div')
	reset3_button.className = 'reset_timer_duration_button'
	reset3_button.id = 'reset_timer_duration_button'
	reset3_button.title = 'Remove timer (double click to clear all fields)'
	reset3_button.onclick = function() {
		document.getElementById('timer_duration_hours').value = '--'
		document.getElementById('timer_duration_mins').value = '--'
		document.getElementById('timer_duration_secs').value = '--'
	}
	reset3_button.ondblclick = reset1_button.ondblclick
	timefields_div.appendChild(reset3_button)

	// 'cancel' button
	var cancel_button = document.createElement('div')
	cancel_button.className = 'gutter_settings_button'
	cancel_button.id = 'cancel_settings_button'
	cancel_button.onclick = function() { close_gutter_settings(); }
	cancel_button.appendChild(document.createTextNode('Cancel'))
	
	// 'ok' button
	var save_button = document.createElement('div')
	save_button.className = 'gutter_settings_button'
	save_button.id = 'save_settings_button'
	save_button.onclick = function() { save_gutter_settings(); }
	save_button.appendChild(document.createTextNode('OK'))

	// final setup = the parent div 'gutter_settings'
	var gutter_settings_div = document.createElement('div')
	gutter_settings_div.className = 'gutter_settings'
	gutter_settings_div.id = 'gutter_settings'
	gutter_settings_div.appendChild(settings_icon_div)
	gutter_settings_div.appendChild(timefields_div)
	gutter_settings_div.appendChild(cancel_button)
	gutter_settings_div.appendChild(save_button)

	// insert in document
	gutter_div.appendChild(gutter_settings_div)
	
	// populate the new gutter_settings div with the current values for this gutter
	populate_gutter_settings()
	
	// add a div over the whole document so that clicking outside of the panel dismisses it
	var overlay_div = document.createElement('div')
	overlay_div.id = 'overlay'
	overlay_div.onclick = function() { close_gutter_settings() }
	var root_div = document.getElementById('root')
	if ( root_div )
		root_div.appendChild(overlay_div)
		
	// add key bindings: enter --> OK , esc --> Cancel
	document.body.onkeypress = function(event) {
		if ( event.keyCode == 13 )
			save_gutter_settings()
		else if ( event.keyCode == 27 )
			close_gutter_settings()
		else
			return true
		return false
	}
	
	// prevent updates during settings
	stop_timers()
}

function populate_gutter_settings()
{
	var gutter_settings_div = document.getElementById('gutter_settings')
	if ( ! gutter_settings_div )
		return;
	var gutter_div = gutter_settings_div.parentNode
	
	// populate the text field values with the right numbers
	var timefields = new Array( 'timer_duration', 'timestamp_start', 'timestamp_end' )
	for ( index in [0,1,2] ) {
		var identifier = timefields[index]
		var value = new TimeStamp(gutter_div.getAttribute(identifier))
		var field_keys = [ 'hours' , 'mins' , 'secs' ]
		var field_values = [ value.hoursString(), value.minutesString(), value.secondsString() ]
		for ( field_index in [0,1,2] ) {
			var field_key = field_keys[field_index]
			var field_string = field_values[field_index]
			// textfield id is of the form 'timer_duration_hours', 'timer_duration_mins',...
			var textfield = document.getElementById( identifier + "_" + field_key )
			if ( textfield != null )
				textfield.value = field_string
		}
	}
}

function close_gutter_settings()
{
	// remove the gutter_settings panel from the document
	var gutter_settings_div = document.getElementById('gutter_settings')
	if ( gutter_settings_div )
		gutter_settings_div.parentNode.removeChild(gutter_settings_div)
	
	// remove the overlay div from the document
	var overlay_div = document.getElementById('overlay')
	if ( overlay_div && overlay_div.parentNode )
		overlay_div.parentNode.removeChild(overlay_div)

	// remove key bidings
	var documentBody = document.body
	delete documentBody.onkeypress

	// resume timer updates
	run_timers()
}

function timestamp_from_time_field(timefield_div)
{
	if ( ! timefield_div.className.match(/time_input/) )
		die_because_timefield_is_not_valid()
	var fields = timefield_div.getElementsByTagName('input')
	if ( fields.length != 3 )
		die_because_number_of_input_fields_is_not_valid()
	var hours = -1
	var mins = -1
	var secs = -1
	for ( var index in [0,1,2] ) {
		var textfield = fields[index]
		if ( textfield.className.match(/hours/) )
			hours = textfield.value
		else if ( textfield.className.match(/mins/) )
			mins = textfield.value
		else if ( textfield.className.match(/secs/) )
			secs = textfield.value
	}
	return new TimeStamp(hours, mins, secs)
}

function save_gutter_settings()
{
	// error checking
	var gutter_settings_div = document.getElementById('gutter_settings')
	if ( ! gutter_settings_div )
		return;
	var gutter_div = gutter_settings_div.parentNode
	if ( ! gutter_div ) {
		close_gutter_settings()
		return;
	}

	// get new values from text fields
	var new_duration = timestamp_from_time_field ( document.getElementById( 'time_input_timer_duration' ) )
	var new_start    = timestamp_from_time_field ( document.getElementById( 'time_input_timestamp_start' ) )
	var new_end      = timestamp_from_time_field ( document.getElementById( 'time_input_timestamp_end' ) )

	// update gutter_div attributes based on fields
	gutter_div.setAttribute('timer_duration'  , new_duration.timeString() )
	gutter_div.setAttribute('timestamp_start' , new_start.timeString() )
	gutter_div.setAttribute('timestamp_end'   , new_end.timeString() )
	
	// clean-up
	gutter_div.removeAttribute('mode');
	close_gutter_settings()
	update_gutter_contents(gutter_div)

}


/* Gutter Icon Clicks */

// click on the checkmark icon --> toggle between 2 styles of bullets if we have a paragraph of class `bullet`
function did_click_gutter_checkmark(event)
{
	var gutter_icon_div = event.target
	var gutter_div = gutter_icon_div.parentNode
	
	var paragraph = gutter_div.parentNode
	var container_body = paragraph.parentNode
	var container = container_body.parentNode
	var p = paragraph.getElementsByTagName('p')[0]
	
	if ( ! p.className.match(/bullet|number|normal|code/) )
		return

	// current state = done --> change to not done
	if ( p.className.match(/done/) ) {
		p_class = p.className.replace('done','')
		p_class = p_class.replace('  ',' ')
		p.className = p_class
		gutter_icon_div.setAttribute('title', 'Mark as done')
	}

	// current state = not done --> change to done
	else {
		p.className = p.className + ' done'
		gutter_icon_div.setAttribute('title', 'Unmark as done')
	}

    if ( bridge_is_available() )
        Bridge.didClickCheckmark(bridge_id_for_node(event.target))
}

// click on the timestamp icon --> set the 'Start' timestamp
function did_click_gutter_timestamp(event)
{
	// disabled for now
	return
	
	gutter_icon_div = event.target
	gutter_div = gutter_icon_div.parentNode
	var paragraph = gutter_div.parentNode
	var container_body = paragraph.parentNode
	var container = container_body.parentNode
	var p = paragraph.getElementsByTagName('p')[0]

	// add checkmark
	if ( p.className.match(/bullet/) &&  ( ! p.className.match(/done/) ) )
		p.className = p.className + ' done'

	// timestamp start
	gutter_div.setAttribute('display', 'start')
	ts = new TimeStamp()
	gutter_div.setAttribute('timestamp_start', ts.timeString() )
	gutter_div.removeAttribute('timestamp_end')
	update_gutter_contents(gutter_div)
	
	// update icons
	remove_icon_from_gutter('timer', gutter_div)
	remove_icon_from_gutter('timestamp', gutter_div)
	remove_icon_from_gutter('checkmark', gutter_div)
	add_icon_to_gutter('gear', gutter_div)
}

// click on the timestamp_end icon = stop the stopwatch --> set the 'End' timestamp
function did_click_gutter_timestamp_end(event)
{
	// disabled for now
	return
	
	gutter_icon_div = event.target
	gutter_div = gutter_icon_div.parentNode

	// timestamp end
	ts = new TimeStamp()
	gutter_div.setAttribute('timestamp_end', ts.timeString() )
	update_gutter_contents(gutter_div)
}

function did_click_gutter_timer(event)
{
	// disabled for now
	return
	
	var gutter_icon_div = event.target
	var gutter_div = gutter_icon_div.parentNode

	// check the contents of the paragraph to see if any value can be had from it
	var paragraph = gutter_div.parentNode
	var p = paragraph.getElementsByTagName('p')[0]
	var duration = timestamp_from_text(p.innerText)

	// default to 5 min
	if ( duration.isDefined() == false )
		duration = new TimeStamp('0:5:0')
	gutter_div.setAttribute('timer_duration', duration.timeString() )

	// other settings
	gutter_div.setAttribute('display', 'duration')
	gutter_div.setAttribute('timer_status', 'pending')
	remove_all_icons_from_gutter(gutter_div)
	add_icon_to_gutter('gear', gutter_div)

	update_gutter_contents(gutter_div)

	// open_settings_for_gutter(gutter_div)
}

function did_click_gutter_gear(event)
{
	// disabled for now
	return
	
	gutter_icon_div = event.target
	gutter_div = gutter_icon_div.parentNode
	open_settings_for_gutter(gutter_div)
}


function did_click_gutter_timer_start(event)
{
	// disabled for now
	return
	
	gutter_icon_div = event.target
	gutter_div = gutter_icon_div.parentNode
	ts = new TimeStamp()
	gutter_div.setAttribute('timestamp_start', ts.timeString() )
	gutter_div.setAttribute('timer_status', 'running')
	update_gutter_contents(gutter_div)
}

function did_click_gutter_timer_pause(event)
{
	// disabled for now
	return
	
	gutter_icon_div = event.target
	gutter_div = gutter_icon_div.parentNode
	gutter_div.setAttribute('timer_status', 'paused')
	update_gutter_contents(gutter_div)
}

function did_click_gutter_timer_resume(event)
{
	// disabled for now
	return
	
	gutter_icon_div = event.target
	gutter_div = gutter_icon_div.parentNode
	var time_left = new TimeStamp(gutter_div.getAttribute('timer_time_left'))
	var now = new TimeStamp()
	var end = now.plusTimeStamp(time_left)
	gutter_div.setAttribute('timestamp_end', end.timeString())
	gutter_div.setAttribute('timer_status', 'running')
	update_gutter_contents(gutter_div)
}



